@@ -1,7 +1,8 @@
-import inspect
-from plumbum.lib import six
+from plumbum.lib import six, getdoc
 from plumbum import local
 
+from abc import abstractmethod
+
 
 class SwitchError(Exception):
     """A general switch related-error (base class of all other switch errors)"""
@@ -39,7 +40,7 @@ class SwitchInfo(object):
             setattr(self, k, v)
 
 def switch(names, argtype = None, argname = None, list = False, mandatory = False, requires = (),
-        excludes = (), help = None, overridable = False, group = "Switches"):
+        excludes = (), help = None, overridable = False, group = "Switches", envname=None):
     """
     A decorator that exposes functions as command-line switches. Usage::
 
@@ -63,6 +64,8 @@ def switch(names, argtype = None, argname = None, list = False, mandatory = Fals
                   but it can be used for clarity. Single-letter names are prefixed by ``-``,
                   while longer names are prefixed by ``--``
 
+    :param envname:   Name of environment variable to extract value from, as alternative to argv
+
     :param argtype: If this function takes an argument, you need to specify its type. The
                     default is ``None``, which means the function takes no argument. The type
                     is more of a "validator" than a real type; it can be any callable object
@@ -131,17 +134,17 @@ def switch(names, argtype = None, argname = None, list = False, mandatory = Fals
 
     def deco(func):
         if argname is None:
-            argspec = inspect.getargspec(func)[0]
+            argspec = six.getfullargspec(func).args
             if len(argspec) == 2:
                 argname2 = argspec[1]
             else:
                 argname2 = "VALUE"
         else:
             argname2 = argname
-        help2 = inspect.getdoc(func) if help is None else help
+        help2 = getdoc(func) if help is None else help
         if not help2:
             help2 = str(func)
-        func._switch_info = SwitchInfo(names = names, argtype = argtype, list = list, func = func,
+        func._switch_info = SwitchInfo(names = names, envname=envname, argtype = argtype, list = list, func = func,
             mandatory = mandatory, overridable = overridable, group = group,
             requires = requires, excludes = excludes, argname = argname2, help = help2)
         return func
@@ -250,10 +253,95 @@ class CountOf(SwitchAttr):
     def __call__(self, inst, v):
         self.__set__(inst, len(v))
 
+#===================================================================================================
+# Decorator for function that adds argument checking
+#===================================================================================================
+
+
+
+class positional(object):
+    """
+    Runs a validator on the main function for a class.    
+    This should be used like this::
+    
+        class MyApp(cli.Application):
+            @cli.positional(cli.Range(1,10), cli.ExistingFile)
+            def main(self, x, *f):
+                # x is a range, f's are all ExistingFile's)
+    
+    Or, Python 3 only::
+    
+        class MyApp(cli.Application):
+            def main(self, x : cli.Range(1,10), *f : cli.ExistingFile):
+                # x is a range, f's are all ExistingFile's)
+        
+        
+    If you do not want to validate on the annotations, use this decorator (
+    even if empty) to override annotation validation.
+    
+    Validators should be callable, and should have a ``.choices()`` function with
+    possible choices. (For future argument completion, for example)
+    
+    Default arguments do not go through the validator.
+    
+    #TODO: Check with MyPy
+    
+    """
+    
+    def __init__(self, *args, **kargs):
+        self.args = args
+        self.kargs = kargs
+    
+    def __call__(self, function):
+        m = six.getfullargspec(function)
+        args_names = list(m.args[1:])
+            
+        positional = [None]*len(args_names)
+        varargs = None
+        
+        for i in range(min(len(positional),len(self.args))):
+            positional[i] = self.args[i]
+        
+        if len(args_names) + 1 == len(self.args):
+            varargs = self.args[-1]
+        
+         # All args are positional, so convert kargs to positional
+        for item in self.kargs:
+            if item == m.varargs:
+                varargs = self.kargs[item]
+            else:
+                positional[args_names.index(item)] = self.kargs[item]
+            
+        function.positional = positional
+        function.positional_varargs = varargs
+        return function
+    
+class Validator(six.ABC):
+    __slots__ = ()
+    
+    @abstractmethod
+    def __call__(self, obj):
+        "Must be implemented for a Validator to work"
+        
+    def choices(self, partial=""):
+        """Should return set of valid choices, can be given optional partial info"""
+        return set()
+        
+    def __repr__(self):
+        """If not overridden, will print the slots as args"""
+        
+        slots = {}
+        for cls in self.__mro__:
+            for prop in getattr(cls, "__slots__", ()):
+                if prop[0] != '_':
+                    slots[prop] = getattr(self, prop)
+        mystrs = ("{0} = {1}".format(name, slots[name]) for name in slots)
+        return "{0}({1})".format(self.__class__.__name__, ", ".join(mystrs))
+    
 #===================================================================================================
 # Switch type validators
 #===================================================================================================
-class Range(object):
+class Range(Validator):
     """
     A switch-type validator that checks for the inclusion of a value in a certain range.
     Usage::
@@ -264,6 +352,8 @@ class Range(object):
     :param start: The minimal value
     :param end: The maximal value
     """
+    __slots__ = ("start", "end")
+    
     def __init__(self, start, end):
         self.start = start
         self.end = end
@@ -274,18 +364,21 @@ class Range(object):
         if obj < self.start or obj > self.end:
             raise ValueError("Not in range [%d..%d]" % (self.start, self.end))
         return obj
+    def choices(self, partial=""):
+        # TODO: Add partial handling
+        return set(range(self.start, self.end+1))
 
-class Set(object):
+class Set(Validator):
     """
     A switch-type validator that checks that the value is contained in a defined
     set of values. Usage::
 
         class MyApp(Application):
-            mode = SwitchAttr(["--mode"], Set("TCP", "UDP", case_insensitive = False))
+            mode = SwitchAttr(["--mode"], Set("TCP", "UDP", case_sensitive = False))
 
     :param values: The set of values (strings)
-    :param case_insensitive: A keyword argument that indicates whether to use case-sensitive
-                             comparison or not. The default is ``True``
+    :param case_sensitive: A keyword argument that indicates whether to use case-sensitive
+                             comparison or not. The default is ``False``
     """
     def __init__(self, *values, **kwargs):
         self.case_sensitive = kwargs.pop("case_sensitive", False)
@@ -300,6 +393,9 @@ class Set(object):
         if obj not in self.values:
             raise ValueError("Expected one of %r" % (list(self.values.values()),))
         return self.values[obj]
+    def choices(self, partial=""):
+        # TODO: Add case sensitive/insensitive parital completion
+        return set(self.values)
 
 class Predicate(object):
     """A wrapper for a single-argument function with pretty printing"""
@@ -309,12 +405,14 @@ class Predicate(object):
         return self.func.__name__
     def __call__(self, val):
         return self.func(val)
+    def choices(self, partial=""):
+        return set()
 
 @Predicate
 def ExistingDirectory(val):
     """A switch-type validator that ensures that the given argument is an existing directory"""
     p = local.path(val)
-    if not p.isdir():
+    if not p.is_dir():
         raise ValueError("%r is not a directory" % (val,))
     return p
 
@@ -322,7 +420,7 @@ def ExistingDirectory(val):
 def ExistingFile(val):
     """A switch-type validator that ensures that the given argument is an existing file"""
     p = local.path(val)
-    if not p.isfile():
+    if not p.is_file():
         raise ValueError("%r is not a file" % (val,))
     return p
 
