@@ -1,97 +1,28 @@
 """
 Terminal-related utilities
+--------------------------
 """
+
+from __future__ import division, print_function, absolute_import
 import sys
 import os
-import platform
-from struct import Struct
 from plumbum import local
+from plumbum.cli.termsize import get_terminal_size
+from plumbum.cli.progress import Progress
 
-
-def get_terminal_size():
-    """
-    Get width and height of console; works on linux, os x, windows and cygwin
-
-    Adapted from https://gist.github.com/jtriley/1108174
-    Originally from: http://stackoverflow.com/questions/566746/how-to-get-console-window-width-in-python
-    """
-    current_os = platform.system()
-    if current_os == 'Windows':
-        size = _get_terminal_size_windows()
-        if not size:
-            # needed for window's python in cygwin's xterm!
-            size = _get_terminal_size_tput()
-    elif current_os in ('Linux', 'Darwin', 'FreeBSD') or current_os.startswith('CYGWIN'):
-        size = _get_terminal_size_linux()
-    
-    if size is None: # we'll assume the standard 80x25 if for any reason we don't know the terminal size
-        size = (80, 25)
-    return size
-
-def _get_terminal_size_windows():
-    try:
-        from ctypes import windll, create_string_buffer
-        STDERR_HANDLE = -12
-        h = windll.kernel32.GetStdHandle(STDERR_HANDLE)
-        csbi_struct = Struct("hhhhHhhhhhh")
-        csbi = create_string_buffer(csbi_struct.size)
-        res = windll.kernel32.GetConsoleScreenBufferInfo(h, csbi)
-        if res:
-            _, _, _, _, _, left, top, right, bottom, _, _ = csbi_struct.unpack(csbi.raw)
-            return right - left + 1, bottom - top + 1
-        return None
-    except Exception:
-        return None
-
-def _get_terminal_size_tput():
-    # get terminal width
-    # src: http://stackoverflow.com/questions/263890/how-do-i-find-the-width-height-of-a-terminal-window
-    try:
-        from plumbum.cmd import tput
-        cols = int(tput('cols'))
-        rows = int(tput('lines'))
-        return (cols, rows)
-    except Exception:
-        return None
-
-def _ioctl_GWINSZ(fd):
-    yx = Struct("hh")
-    try:
-        import fcntl
-        import termios
-        return yx.unpack(fcntl.ioctl(fd, termios.TIOCGWINSZ, '1234'))
-    except Exception:
-        return None
-
-def _get_terminal_size_linux():
-    cr = _ioctl_GWINSZ(0) or _ioctl_GWINSZ(1) or _ioctl_GWINSZ(2)
-    if not cr:
-        try:
-            fd = os.open(os.ctermid(), os.O_RDONLY)
-            cr = _ioctl_GWINSZ(fd)
-            os.close(fd)
-        except Exception:
-            pass
-    if not cr:
-        try:
-            cr = (int(os.environ['LINES']), int(os.environ['COLUMNS']))
-        except Exception:
-            return None
-    return cr[1], cr[0]
-
-def readline(message = ""):                          
+def readline(message = ""):
     """Gets a line of input from the user (stdin)"""
     sys.stdout.write(message)
     return sys.stdin.readline()
 
 def ask(question, default = None):
     """
-    Presents the user with a yes/no question. 
-    
+    Presents the user with a yes/no question.
+
     :param question: The question to ask
-    :param default: If ``None``, the user must answer. If ``True`` or ``False``, lack of response is 
+    :param default: If ``None``, the user must answer. If ``True`` or ``False``, lack of response is
                     interpreted as the default option
-    
+
     :returns: the user's choice
     """
     question = question.rstrip().rstrip("?").rstrip() + "?"
@@ -101,7 +32,7 @@ def ask(question, default = None):
         question += " [Y/n] "
     else:
         question += " [y/N] "
-    
+
     while True:
         try:
             answer = readline(question).strip().lower()
@@ -118,21 +49,21 @@ def ask(question, default = None):
 
 def choose(question, options, default = None):
     """Prompts the user with a question and a set of options, from which the user need choose.
-    
+
     :param question: The question to ask
-    :param options: A set of options. It can be a list (of strings or two-tuples, mapping text 
+    :param options: A set of options. It can be a list (of strings or two-tuples, mapping text
                     to returned-object) or a dict (mapping text to returned-object).``
     :param default: If ``None``, the user must answer. Otherwise, lack of response is interpreted
                     as this answer
-    
+
     :returns: The user's choice
-    
+
     Example::
-    
+
         ans = choose("What is your favorite color?", ["blue", "yellow", "green"], default = "yellow")
         # `ans` will be one of "blue", "yellow" or "green"
 
-        ans = choose("What is your favorite color?", 
+        ans = choose("What is your favorite color?",
                 {"blue" : 0x0000ff, "yellow" : 0xffff00 , "green" : 0x00ff00}, default = 0x00ff00)
         # this will display "blue", "yellow" and "green" but return a numerical value
     """
@@ -204,7 +135,7 @@ def prompt(question, type = int, default = NotImplemented, validator = lambda va
         return ans
 
 def hexdump(data_or_stream, bytes_per_line = 16, aggregate = True):
-    """Convert the given bytes (or a stream with a buffering ``read()`` method) to hexdump-formatted lines, 
+    """Convert the given bytes (or a stream with a buffering ``read()`` method) to hexdump-formatted lines,
     with possible aggregation of identical lines. Returns a generator of formatted lines.
     """
     if hasattr(data_or_stream, "read"):
@@ -235,7 +166,7 @@ def hexdump(data_or_stream, bytes_per_line = 16, aggregate = True):
 
 def pager(rows, pagercmd = None):
     """Opens a pager (e.g., ``less``) to display the given text. Requires a terminal.
-    
+
     :param rows: a ``bytes`` or a list/iterator of "rows" (``bytes``)
     :param pagercmd: the pager program to run. Defaults to ``less -RSin``
     """
