@@ -1,4 +1,3 @@
-from __future__ import with_statement
 import time
 import atexit
 import heapq
@@ -17,42 +16,6 @@ except ImportError:
     from cStringIO import StringIO
 
 
-if not hasattr(Popen, "kill"):
-    # python 2.5 compatibility
-    import os
-    import sys
-    import signal
-    if IS_WIN32:
-        import _subprocess
-
-        def _Popen_terminate(self):
-            """taken from subprocess.py of python 2.7"""
-            try:
-                _subprocess.TerminateProcess(self._handle, 1)
-            except OSError:
-                ex = sys.exc_info()[1]
-                # ERROR_ACCESS_DENIED (winerror 5) is received when the
-                # process already died.
-                if ex.winerror != 5:
-                    raise
-                rc = _subprocess.GetExitCodeProcess(self._handle)
-                if rc == _subprocess.STILL_ACTIVE:
-                    raise
-                self.returncode = rc
-
-        Popen.kill = _Popen_terminate
-        Popen.terminate = _Popen_terminate
-    else:
-        def _Popen_kill(self):
-            os.kill(self.pid, signal.SIGKILL)
-        def _Popen_terminate(self):
-            os.kill(self.pid, signal.SIGTERM)
-        def _Popen_send_signal(self, sig):
-            os.kill(self.pid, sig)
-        Popen.kill = _Popen_kill
-        Popen.terminate = _Popen_kill
-        Popen.send_signal = _Popen_send_signal
-
 #===================================================================================================
 # utility functions
 #===================================================================================================
@@ -138,7 +101,7 @@ class ProcessTimedOut(Exception):
         Exception.__init__(self, msg, argv)
         self.argv = argv
 
-class CommandNotFound(Exception):
+class CommandNotFound(AttributeError):
     """Raised by :func:`local.which <plumbum.machines.local.LocalMachine.which>` and
     :func:`RemoteMachine.which <plumbum.machines.remote.RemoteMachine.which>` when a
     command was not found in the system's ``PATH``"""
