@@ -1,4 +1,3 @@
-from __future__ import with_statement
 import os
 import sys
 import subprocess
@@ -13,9 +12,10 @@ from contextlib import contextmanager
 from plumbum.path.remote import RemotePath
 from plumbum.commands import CommandNotFound, ConcreteCommand
 from plumbum.machines.session import ShellSession
-from plumbum.lib import ProcInfo, IS_WIN32, six
+from plumbum.lib import ProcInfo, IS_WIN32, six, StaticProperty
 from plumbum.commands.daemons import win32_daemonize, posix_daemonize
 from plumbum.commands.processes import iter_lines
+from plumbum.machines.base import BaseMachine
 from plumbum.machines.env import BaseEnv
 
 if sys.version_info >= (3, 2):
@@ -94,8 +94,6 @@ class LocalCommand(ConcreteCommand):
     def __init__(self, executable, encoding = "auto"):
         ConcreteCommand.__init__(self, executable,
             local.encoding if encoding == "auto" else encoding)
-    def __repr__(self):
-        return "LocalCommand(%r)" % (self.executable,)
 
     @property
     def machine(self):
@@ -104,14 +102,16 @@ class LocalCommand(ConcreteCommand):
     def popen(self, args = (), cwd = None, env = None, **kwargs):
         if isinstance(args, six.string_types):
             args = (args,)
-        return local._popen(self.executable, self.formulate(0, args),
+        return self.machine._popen(self.executable, self.formulate(0, args),
             cwd = self.cwd if cwd is None else cwd, env = self.env if env is None else env,
             **kwargs)
 
 #===================================================================================================
 # Local Machine
 #===================================================================================================
-class LocalMachine(object):
+
+
+class LocalMachine(BaseMachine):
     """The *local machine* (a singleton object). It serves as an entry point to everything
     related to the local machine, such as working directory and environment manipulation,
     command creation, etc.
@@ -122,8 +122,10 @@ class LocalMachine(object):
     * ``env`` - the local environment
     * ``encoding`` - the local machine's default encoding (``sys.getfilesystemencoding()``)
     """
-    cwd = LocalWorkdir()
+
+    cwd = StaticProperty(LocalWorkdir)
     env = LocalEnv()
+
     encoding = sys.getfilesystemencoding()
     uname = platform.uname()[0]
 
@@ -192,6 +194,7 @@ class LocalMachine(object):
 
             ls = local["ls"]
         """
+
         if isinstance(cmd, LocalPath):
             return LocalCommand(cmd)
         elif not isinstance(cmd, RemotePath):
@@ -204,23 +207,12 @@ class LocalMachine(object):
         else:
             raise TypeError("cmd must not be a RemotePath: %r" % (cmd,))
 
-    def __contains__(self, cmd):
-        """Tests for the existance of the command, e.g., ``"ls" in plumbum.local``.
-        ``cmd`` can be anything acceptable by ``__getitem__``.
-        """
-        try:
-            self[cmd]
-        except CommandNotFound:
-            return False
-        else:
-            return True
-
     def _popen(self, executable, argv, stdin = PIPE, stdout = PIPE, stderr = PIPE,
             cwd = None, env = None, new_session = False, **kwargs):
         if new_session:
             if has_new_subprocess:
                 kwargs["start_new_session"] = True
-            elif subprocess.mswindows:
+            elif IS_WIN32:
                 kwargs["creationflags"] = kwargs.get("creationflags", 0) | subprocess.CREATE_NEW_PROCESS_GROUP
             else:
                 def preexec_fn(prev_fn = kwargs.get("preexec_fn", lambda: None)):
@@ -228,7 +220,7 @@ class LocalMachine(object):
                     prev_fn()
                 kwargs["preexec_fn"] = preexec_fn
 
-        if subprocess.mswindows and "startupinfo" not in kwargs and stdin not in (sys.stdin, None):
+        if IS_WIN32 and "startupinfo" not in kwargs and stdin not in (sys.stdin, None):
             from plumbum.machines._windows import get_pe_subsystem, IMAGE_SUBSYSTEM_WINDOWS_CUI
             subsystem = get_pe_subsystem(str(executable))
 
@@ -244,7 +236,7 @@ class LocalMachine(object):
                     sui.wShowWindow = subprocess.SW_HIDE  # @UndefinedVariable
 
         if not has_new_subprocess and "close_fds" not in kwargs:
-            if subprocess.mswindows and (stdin is not None or stdout is not None or stderr is not None):
+            if IS_WIN32 and (stdin is not None or stdout is not None or stderr is not None):
                 # we can't close fds if we're on windows and we want to redirect any std handle
                 kwargs["close_fds"] = False
             else:
@@ -268,7 +260,8 @@ class LocalMachine(object):
         proc.argv = argv
         return proc
 
-    def daemonic_popen(self, command, cwd = "/"):
+
+    def daemonic_popen(self, command, cwd = "/", stdout=None, stderr=None, append=True):
         """
         On POSIX systems:
 
@@ -287,9 +280,9 @@ class LocalMachine(object):
         .. versionadded:: 1.3
         """
         if IS_WIN32:
-            return win32_daemonize(command, cwd)
+            return win32_daemonize(command, cwd, stdout, stderr, append)
         else:
-            return posix_daemonize(command, cwd)
+            return posix_daemonize(command, cwd, stdout, stderr, append)
 
     if IS_WIN32:
         def list_processes(self):
@@ -300,16 +293,16 @@ class LocalMachine(object):
             """
             import csv
             tasklist = local["tasklist"]
-            lines = tasklist("/V", "/FO", "CSV").encode("utf8").splitlines()
+            lines = tasklist("/V", "/FO", "CSV").splitlines()
             rows = csv.reader(lines)
-            header = rows.next()
+            header = next(rows)
             imgidx = header.index('Image Name')
             pididx = header.index('PID')
             statidx = header.index('Status')
             useridx = header.index('User Name')
             for row in rows:
-                yield ProcInfo(int(row[pididx]), row[useridx].decode("utf8"),
-                    row[statidx].decode("utf8"), row[imgidx].decode("utf8"))
+                yield ProcInfo(int(row[pididx]), row[useridx],
+                    row[statidx], row[imgidx])
     else:
         def list_processes(self):
             """
