@@ -1,10 +1,10 @@
-from __future__ import with_statement
 import re
 from contextlib import contextmanager
 from plumbum.commands import CommandNotFound, shquote, ConcreteCommand
 from plumbum.lib import _setdoc, ProcInfo, six
 from plumbum.machines.local import LocalPath
 from tempfile import NamedTemporaryFile
+from plumbum.machines.base import BaseMachine
 from plumbum.machines.env import BaseEnv
 from plumbum.path.remote import RemotePath, RemoteWorkdir, StatRes
 
@@ -113,6 +113,9 @@ class RemoteCommand(ConcreteCommand):
         return "RemoteCommand(%r, %r)" % (self.remote, self.executable)
     def popen(self, args = (), **kwargs):
         return self.remote.popen(self[args], **kwargs)
+    def nohup(self, cwd='.', stdout='nohup.out', stderr=None, append=True):
+        """Runs a command detached."""
+        return machine.nohup(self, cwd, stdout, stderr, append)
 
 class ClosedRemoteMachine(Exception):
     pass
@@ -127,7 +130,7 @@ class ClosedRemote(object):
         raise ClosedRemoteMachine("%r has been closed" % (self._obj,))
 
 
-class BaseRemoteMachine(object):
+class BaseRemoteMachine(BaseMachine):
     """Represents a *remote machine*; serves as an entry point to everything related to that
     remote machine, such as working directory and environment manipulation, command creation,
     etc.
@@ -142,14 +145,16 @@ class BaseRemoteMachine(object):
 
     # allow inheritors to override the RemoteCommand class
     RemoteCommand = RemoteCommand
-
+    
+    @property
+    def cwd(self):
+        return RemoteWorkdir(self)
 
     def __init__(self, encoding = "utf8", connect_timeout = 10, new_session = False):
         self.encoding = encoding
         self.connect_timeout = connect_timeout
         self._session = self.session(new_session = new_session)
         self.uname = self._get_uname()
-        self.cwd = RemoteWorkdir(self)
         self.env = RemoteEnv(self)
         self._python = None
 
@@ -236,17 +241,6 @@ class BaseRemoteMachine(object):
         else:
             raise TypeError("cmd must not be a LocalPath: %r" % (cmd,))
 
-    def __contains__(self, cmd):
-        """Tests for the existance of the command, e.g., ``"ls" in remote_machine``.
-        ``cmd`` can be anything acceptable by ``__getitem__``.
-        """
-        try:
-            self[cmd]
-        except CommandNotFound:
-            return False
-        else:
-            return True
-
     @property
     def python(self):
         """A command that represents the default remote python interpreter"""
@@ -327,13 +321,13 @@ class BaseRemoteMachine(object):
         return matches
 
     def _path_getuid(self, fn):
-        stat_cmd = "stat -c '%u,%U' " if self.uname != 'Darwin' else "stat -f '%u,%Su' "
+        stat_cmd = "stat -c '%u,%U' " if self.uname not in ('Darwin', 'FreeBSD') else "stat -f '%u,%Su' "
         return self._session.run(stat_cmd + shquote(fn))[1].strip().split(",")
     def _path_getgid(self, fn):
-        stat_cmd = "stat -c '%g,%G' " if self.uname != 'Darwin' else "stat -f '%g,%Sg' "
+        stat_cmd = "stat -c '%g,%G' " if self.uname not in ('Darwin', 'FreeBSD') else "stat -f '%g,%Sg' "
         return self._session.run(stat_cmd + shquote(fn))[1].strip().split(",")
     def _path_stat(self, fn):
-        if self.uname != 'Darwin':
+        if self.uname not in ('Darwin', 'FreeBSD'):
             stat_cmd = "stat -c '%F,%f,%i,%d,%h,%u,%g,%s,%X,%Y,%Z' "
         else:
             stat_cmd = "stat -f '%HT,%Xp,%i,%d,%l,%u,%g,%z,%a,%m,%c' "
