@@ -1,11 +1,13 @@
+from __future__ import absolute_import
+
 import itertools
 import operator
 import os
+from plumbum.lib import six
+from abc import abstractmethod, abstractproperty
+import warnings
 
-try:
-    reduce
-except NameError:
-    from functools import reduce
+from functools import reduce
 
 
 class FSUser(int):
@@ -18,13 +20,12 @@ class FSUser(int):
         self.name = name
         return self
 
-class Path(object):
+class Path(str, six.ABC):
     """An abstraction over file system paths. This class is abstract, and the two implementations
     are :class:`LocalPath <plumbum.machines.local.LocalPath>` and
     :class:`RemotePath <plumbum.path.remote.RemotePath>`.
     """
 
-    __slots__ = []
     CASE_SENSITIVE = True
 
     def __repr__(self):
@@ -68,8 +69,9 @@ class Path(object):
         return bool(str(self))
     __bool__ = __nonzero__
 
+    @abstractmethod
     def _form(self, *parts):
-        raise NotImplementedError()
+        pass
 
     def up(self, count = 1):
         """Go up in ``count`` directories (the default is 1)"""
@@ -78,8 +80,8 @@ class Path(object):
         """traverse all (recursive) sub-elements under this directory, that match the given filter.
         By default, the filter accepts everything; you can provide a custom filter function that
         takes a path as an argument and returns a boolean
-        
-        :param filter: the filter (predicate function) for matching results. Only paths matching 
+
+        :param filter: the filter (predicate function) for matching results. Only paths matching
                        this predicate are returned. Defaults to everything.
         :param dir_filter: the filter (predicate function) for matching directories. Only directories
                            matching this predicate are recursed into. Defaults to everything.
@@ -87,150 +89,194 @@ class Path(object):
         for p in self.list():
             if filter(p):
                 yield p
-            if p.isdir() and dir_filter(p):
+            if p.is_dir() and dir_filter(p):
                 for p2 in p.walk(filter, dir_filter):
                     yield p2
 
+    @abstractproperty
+    def name(self):
+        """The basename component of this path"""
+
     @property
     def basename(self):
-        """The basename component of this path"""
-        raise NotImplementedError()
-    @property
+        """Included for compatibility with older Plumbum code"""
+        warnings.warn("Use .name instead", DeprecationWarning)
+        return self.name
+
+    @abstractproperty
+    def stem(self):
+        """The name without an extension, or the last component of the path"""
+
+    @abstractproperty
     def dirname(self):
         """The dirname component of this path"""
-        raise NotImplementedError()
-        
-    @property
+
+    @abstractproperty
+    def root(self):
+        """The root of the file tree (`/` on Unix)"""
+
+    @abstractproperty
+    def drive(self):
+        """The drive letter (on Windows)"""
+
+    @abstractproperty
     def suffix(self):
         """The suffix of this file"""
-        raise NotImplementedError()
-    @property
+    @abstractproperty
     def suffixes(self):
         """This is a list of all suffixes"""
-        raise NotImplementedError()
-        
-    @property
+
+    @abstractproperty
     def uid(self):
         """The user that owns this path. The returned value is a :class:`FSUser <plumbum.path.FSUser>`
         object which behaves like an ``int`` (as expected from ``uid``), but it also has a ``.name``
         attribute that holds the string-name of the user"""
-        raise NotImplementedError()
-    @property
+
+    @abstractproperty
     def gid(self):
         """The group that owns this path. The returned value is a :class:`FSUser <plumbum.path.FSUser>`
         object which behaves like an ``int`` (as expected from ``gid``), but it also has a ``.name``
         attribute that holds the string-name of the group"""
-        raise NotImplementedError()
 
+    @abstractmethod
+    def as_uri(self, scheme=None):
+        """Returns a universal resource identifier. Use ``scheme`` to force a scheme."""
+
+    @abstractmethod
     def _get_info(self):
-        raise NotImplementedError()
+        pass
+    @abstractmethod
     def join(self, *parts):
         """Joins this path with any number of paths"""
-        raise NotImplementedError()
+    @abstractmethod
     def list(self):
         """Returns the files in this directory"""
-        raise NotImplementedError()
-    def isdir(self):
+    @abstractmethod
+    def iterdir(self):
+        """Returns an iterator over the directory. Might be slightly faster on Python 3.5 than .list()"""
+    @abstractmethod
+    def is_dir(self):
         """Returns ``True`` if this path is a directory, ``False`` otherwise"""
-        raise NotImplementedError()
-    def isfile(self):
+    def isdir(self):
+        """Included for compatibility with older Plumbum code"""
+        warnings.warn("Use .is_dir() instead", DeprecationWarning)
+        return self.is_dir()
+    @abstractmethod
+    def is_file(self):
         """Returns ``True`` if this path is a regular file, ``False`` otherwise"""
-        raise NotImplementedError()
+    def isfile(self):
+        """Included for compatibility with older Plumbum code"""
+        warnings.warn("Use .is_file() instead", DeprecationWarning)
+        return self.is_file()
     def islink(self):
+        """Included for compatibility with older Plumbum code"""
+        warnings.warn("Use is_symlink instead", DeprecationWarning)
+        return self.is_symlink()
+    @abstractmethod
+    def is_symlink(self):
         """Returns ``True`` if this path is a symbolic link, ``False`` otherwise"""
-        raise NotImplementedError()
+    @abstractmethod
     def exists(self):
         """Returns ``True`` if this path exists, ``False`` otherwise"""
-        raise NotImplementedError()
+    @abstractmethod
     def stat(self):
-        raise NotImplementedError()
+        """Returns the os.stats for a file"""
+        pass
+    @abstractmethod
     def with_name(self, name):
         """Returns a path with the name replaced"""
-        raise NotImplementedError()
+    @abstractmethod
     def with_suffix(self, suffix, depth=1):
         """Returns a path with the suffix replaced. Up to last ``depth`` suffixes will be
         replaces. None will replace all suffixes. If there are less than ``depth`` suffixes,
         this will replace all suffixes. ``.tar.gz`` is an example where ``depth=2`` or
         ``depth=None`` is useful"""
-        raise NotImplementedError()
+
+    def preferred_suffix(self, suffix):
+        """Adds a suffix if one does not currently exist (otherwise, no change). Useful
+        for loading files with a default suffix"""
+        if len(self.suffixes) > 0:
+            return self
+        else:
+            return self.with_suffix(suffix)
+    @abstractmethod
     def glob(self, pattern):
         """Returns a (possibly empty) list of paths that matched the glob-pattern under this path"""
-        raise NotImplementedError()
+    @abstractmethod
     def delete(self):
         """Deletes this path (recursively, if a directory)"""
-        raise NotImplementedError()
+    @abstractmethod
     def move(self, dst):
         """Moves this path to a different location"""
-        raise NotImplementedError()
     def rename(self, newname):
         """Renames this path to the ``new name`` (only the basename is changed)"""
         return self.move(self.up() / newname)
+    @abstractmethod
     def copy(self, dst, override = False):
         """Copies this path (recursively, if a directory) to the destination path"""
-        raise NotImplementedError()
+    @abstractmethod
     def mkdir(self):
         """Creates a directory at this path; if the directory already exists, silently ignore"""
-        raise NotImplementedError()
+    @abstractmethod
     def open(self, mode = "r"):
         """opens this path as a file"""
-        raise NotImplementedError()
+    @abstractmethod
     def read(self, encoding=None):
         """returns the contents of this file. By default the data is binary (``bytes``), but you can
         specify the encoding, e.g., ``'latin1'`` or ``'utf8'``"""
-        raise NotImplementedError()
+    @abstractmethod
     def write(self, data, encoding=None):
-        """writes the given data to this file. By default the data is expected to be binary (``bytes``), 
+        """writes the given data to this file. By default the data is expected to be binary (``bytes``),
         but you can specify the encoding, e.g., ``'latin1'`` or ``'utf8'``"""
-        raise NotImplementedError()
+    @abstractmethod
     def chown(self, owner = None, group = None, recursive = None):
         """Change ownership of this path.
 
         :param owner: The owner to set (either ``uid`` or ``username``), optional
-        :param owner: The group to set (either ``gid`` or ``groupname``), optional
+        :param group: The group to set (either ``gid`` or ``groupname``), optional
         :param recursive: whether to change ownership of all contained files and subdirectories.
                           Only meaningful when ``self`` is a directory. If ``None``, the value
                           will default to ``True`` if ``self`` is a directory, ``False`` otherwise.
         """
-        raise NotImplementedError()
+    @abstractmethod
     def chmod(self, mode):
         """Change the mode of path to the numeric mode.
 
         :param mode: file mode as for os.chmod
         """
-        raise NotImplementedError()
 
     @staticmethod
     def _access_mode_to_flags(mode, flags = {"f" : os.F_OK, "w" : os.W_OK, "r" : os.R_OK, "x" : os.X_OK}):
         if isinstance(mode, str):
             mode = reduce(operator.or_, [flags[m] for m in mode.lower()], 0)
         return mode
-    
+
+    @abstractmethod
     def access(self, mode = 0):
         """Test file existence or permission bits
-        
-        :param mode: a bitwise-or of access bits, or a string-representation thereof: 
-                     ``'f'``, ``'x'``, ``'r'``, ``'w'`` for ``os.F_OK``, ``os.X_OK``, 
+
+        :param mode: a bitwise-or of access bits, or a string-representation thereof:
+                     ``'f'``, ``'x'``, ``'r'``, ``'w'`` for ``os.F_OK``, ``os.X_OK``,
                      ``os.R_OK``, ``os.W_OK``
         """
-        raise NotImplementedError()
 
+    @abstractmethod
     def link(self, dst):
         """Creates a hard link from ``self`` to ``dst``
 
         :param dst: the destination path
         """
-        raise NotImplementedError()
 
+    @abstractmethod
     def symlink(self, dst):
         """Creates a symbolic link from ``self`` to ``dst``
 
         :param dst: the destination path
         """
-        raise NotImplementedError()
 
+    @abstractmethod
     def unlink(self):
         """Deletes a symbolic link"""
-        raise NotImplementedError()
 
     def split(self):
         """Splits the path on directory separators, yielding a list of directories, e.g,
@@ -239,10 +285,15 @@ class Path(object):
         parts = []
         path = self
         while path != path.dirname:
-            parts.append(path.basename)
+            parts.append(path.name)
             path = path.dirname
         return parts[::-1]
 
+    @property
+    def parts(self):
+        """Splits the directory into parts, including the base directroy, returns a tuple"""
+        return tuple([self.root] + self.split())
+
     def relative_to(self, source):
         """Computes the "relative path" require to get from ``source`` to ``self``. They satisfy the invariant
         ``source_path + (target_path - source_path) == target_path``. For example::
@@ -265,16 +316,28 @@ class Path(object):
         """Same as ``self.relative_to(other)``"""
         return self.relative_to(other)
 
+    def _glob(self, pattern, fn):
+        """Applies a glob string or list/tuple/iterable to the current path, using ``fn``"""
+        if isinstance(pattern, str):
+            return fn(pattern)
+        else:
+            results = []
+            for single_pattern in pattern:
+                results.extend(fn(single_pattern))
+            return sorted(list(set(results)))
+
+
 
 class RelativePath(object):
     """
     Relative paths are the "delta" required to get from one path to another.
     Note that relative path do not point at anything, and thus are not paths.
-    Therefore they are system agnostic (but closed under addition) 
+    Therefore they are system agnostic (but closed under addition)
     Paths are always absolute and point at "something", whether existent or not.
-    
+
     Relative paths are created by subtracting paths (``Path.relative_to``)
     """
+
     def __init__(self, parts):
         self.parts = parts
     def __str__(self):
@@ -305,10 +368,10 @@ class RelativePath(object):
     def __nonzero__(self):
         return bool(str(self))
     __bool__ = __nonzero__
-    
+
     def up(self, count = 1):
         return RelativePath(self.parts[:-count])
-    
+
     def __radd__(self, path):
         return path.join(*self.parts)
 
