@@ -1,4 +1,3 @@
-from __future__ import with_statement
 import os
 import sys
 import glob
@@ -22,6 +21,14 @@ except ImportError:
     def getgrnam(x):
         raise OSError("`getgrnam` not supported")
 
+try: # Py3
+    import urllib.parse as urlparse
+    import urllib.request as urllib 
+except ImportError:
+    import urlparse
+    import urllib
+
+
 logger = logging.getLogger("plumbum.local")
 
 
@@ -31,21 +38,23 @@ logger = logging.getLogger("plumbum.local")
 class LocalPath(Path):
     """The class implementing local-machine paths"""
 
-    __slots__ = ["_path"]
     CASE_SENSITIVE = not IS_WIN32
 
-    def __init__(self, *parts):
+    def __new__(cls, *parts):
+        if len(parts) == 1 and \
+                isinstance(parts[0], cls) and \
+                not isinstance(parts[0], LocalWorkdir):
+            return parts[0]
         if not parts:
             raise TypeError("At least one path part is require (none given)")
         if any(isinstance(path, RemotePath) for path in parts):
             raise TypeError("LocalPath cannot be constructed from %r" % (parts,))
-        self._path = os.path.normpath(os.path.join(*(str(p) for p in parts)))
-    def __new__(cls, *parts):
-        if len(parts) == 1 and isinstance(parts[0], cls):
-            return parts[0]
-        return object.__new__(cls)
-    def __str__(self):
-        return self._path
+        self = super(LocalPath, cls).__new__(cls, os.path.normpath(os.path.join(*(str(p) for p in parts))))
+        return self
+    @property
+    def _path(self):
+        return str(self)
+
     def _get_info(self):
         return self._path
     def __getstate__(self):
@@ -56,7 +65,7 @@ class LocalPath(Path):
 
     @property
     @_setdoc(Path)
-    def basename(self):
+    def name(self):
         return os.path.basename(str(self))
 
     @property
@@ -64,6 +73,22 @@ class LocalPath(Path):
     def dirname(self):
         return LocalPath(os.path.dirname(str(self)))
 
+    @property
+    @_setdoc(Path)
+    def suffix(self):
+        return os.path.splitext(str(self))[1]
+
+    @property
+    def suffixes(self):
+        exts = []
+        base = str(self)
+        while True:
+            base, ext = os.path.splitext(base)
+            if ext:
+                exts.append(ext)
+            else:
+                return list(reversed(exts))
+
     @property
     @_setdoc(Path)
     def uid(self):
@@ -85,17 +110,24 @@ class LocalPath(Path):
     @_setdoc(Path)
     def list(self):
         return [self / fn for fn in os.listdir(str(self))]
+        
+    @_setdoc(Path)
+    def iterdir(self):
+        try:
+            return (self.__class__(fn.name) for fn in os.scandir(str(self)))
+        except NameError:
+            return (self / fn for fn in os.listdir(str(self)))
 
     @_setdoc(Path)
-    def isdir(self):
+    def is_dir(self):
         return os.path.isdir(str(self))
 
     @_setdoc(Path)
-    def isfile(self):
+    def is_file(self):
         return os.path.isfile(str(self))
 
     @_setdoc(Path)
-    def islink(self):
+    def is_symlink(self):
         return os.path.islink(str(self))
 
     @_setdoc(Path)
@@ -106,15 +138,35 @@ class LocalPath(Path):
     def stat(self):
         return os.stat(str(self))
 
+    @_setdoc(Path)
+    def with_name(self, name):
+        return LocalPath(self.dirname) / name
+
+    @property
+    @_setdoc(Path)
+    def stem(self):
+        return self.name.rsplit(os.path.extsep)[0]
+
+    @_setdoc(Path)
+    def with_suffix(self, suffix, depth=1):
+        if (suffix and not suffix.startswith(os.path.extsep) or suffix == os.path.extsep):
+            raise ValueError("Invalid suffix %r" % (suffix))
+        name = self.name
+        depth = len(self.suffixes) if depth is None else min(depth, len(self.suffixes))
+        for i in range(depth):
+            name, ext = os.path.splitext(name)
+        return LocalPath(self.dirname) / (name + suffix)
+
     @_setdoc(Path)
     def glob(self, pattern):
-        return [LocalPath(fn) for fn in glob.glob(str(self / pattern))]
+        fn = lambda pat: [LocalPath(m) for m in glob.glob(str(self / pat))]
+        return self._glob(pattern, fn) 
 
     @_setdoc(Path)
     def delete(self):
         if not self.exists():
             return
-        if self.isdir():
+        if self.is_dir():
             shutil.rmtree(str(self))
         else:
             try:
@@ -139,9 +191,12 @@ class LocalPath(Path):
         dst = LocalPath(dst)
         if override:
             dst.delete()
-        if self.isdir():
+        if self.is_dir():
             shutil.copytree(str(self), str(dst))
         else:
+            dst_dir = LocalPath(dst).dirname
+            if not dst_dir.exists():
+                dst_dir.mkdir()
             shutil.copy2(str(self), str(dst))
         return dst
 
@@ -161,12 +216,17 @@ class LocalPath(Path):
         return open(str(self), mode)
 
     @_setdoc(Path)
-    def read(self):
-        with self.open() as f:
-            return f.read()
+    def read(self, encoding=None):
+        with self.open("rb") as f:
+            data = f.read()
+            if encoding:
+                data = data.decode(encoding)
+            return data
 
     @_setdoc(Path)
-    def write(self, data):
+    def write(self, data, encoding=None):
+        if encoding:
+            data = data.encode(encoding)
         with self.open("wb") as f:
             f.write(data)
 
@@ -177,7 +237,7 @@ class LocalPath(Path):
         uid = self.uid if owner is None else (owner if isinstance(owner, int) else getpwnam(owner)[2])
         gid = self.gid if group is None else (group if isinstance(group, int) else getgrnam(group)[2])
         os.chown(str(self), uid, gid)
-        if recursive or (recursive is None and self.isdir()):
+        if recursive or (recursive is None and self.is_dir()):
             for subpath in self.walk():
                 os.chown(str(subpath), uid, gid)
 
@@ -200,7 +260,7 @@ class LocalPath(Path):
         else:
             from plumbum.machines.local import local
             # windows: use mklink
-            if self.isdir():
+            if self.is_dir():
                 local["cmd"]("/C", "mklink", "/D", "/H", str(dst), str(self))
             else:
                 local["cmd"]("/C", "mklink", "/H", str(dst), str(self))
@@ -214,7 +274,7 @@ class LocalPath(Path):
         else:
             from plumbum.machines.local import local
             # windows: use mklink
-            if self.isdir():
+            if self.is_dir():
                 local["cmd"]("/C", "mklink", "/D", str(dst), str(self))
             else:
                 local["cmd"]("/C", "mklink", str(dst), str(self))
@@ -222,24 +282,41 @@ class LocalPath(Path):
     @_setdoc(Path)
     def unlink(self):
         try:
-            os.unlink(str(self))
+            if hasattr(os, "symlink") or not self.is_dir():
+                os.unlink(str(self))
+            else:
+                # windows: use rmdir for directories and directory symlinks
+                os.rmdir(str(self))
         except OSError:
             # file might already been removed (a race with other threads/processes)
             _, ex, _ = sys.exc_info()
             if ex.errno != errno.ENOENT:
                 raise
 
+    @_setdoc(Path)
+    def as_uri(self, scheme='file'):
+        return urlparse.urljoin(str(scheme)+':', urllib.pathname2url(str(self)))
+
+    @property
+    @_setdoc(Path)
+    def drive(self):
+        return os.path.splitdrive(str(self))[0]
+
+    @property
+    @_setdoc(Path)
+    def root(self):
+        return os.path.sep
+    
+
 
 class LocalWorkdir(LocalPath):
     """Working directory manipulator"""
 
-    __slots__ = []
-    def __init__(self):
-        LocalPath.__init__(self, os.getcwd())
+
     def __hash__(self):
         raise TypeError("unhashable type")
     def __new__(cls):
-        return object.__new__(cls)
+        return super(LocalWorkdir, cls).__new__(cls, os.getcwd())
 
     def chdir(self, newdir):
         """Changes the current working directory to the given one
@@ -250,7 +327,7 @@ class LocalWorkdir(LocalPath):
             raise TypeError("newdir cannot be %r" % (newdir,))
         logger.debug("Chdir to %s", newdir)
         os.chdir(str(newdir))
-        self._path = os.path.normpath(os.getcwd())
+        return self.__class__()
     def getpath(self):
         """Returns the current working directory as a ``LocalPath`` object"""
         return LocalPath(self._path)
@@ -262,10 +339,9 @@ class LocalWorkdir(LocalPath):
         :param newdir: The destination director (a string or a ``LocalPath``)
         """
         prev = self._path
-        self.chdir(newdir)
+        newdir = self.chdir(newdir)
         try:
-            yield
+            yield newdir 
         finally:
             self.chdir(prev)
 
-
