@@ -1,10 +1,13 @@
-from __future__ import with_statement
 import errno
 from contextlib import contextmanager
 from plumbum.path.base import Path, FSUser
 from plumbum.lib import _setdoc, six
 from plumbum.commands import shquote
 
+try: # Py3
+    import urllib.request as urllib 
+except ImportError:
+    import urllib
 
 class StatRes(object):
     """POSIX-like stat result"""
@@ -27,14 +30,13 @@ class StatRes(object):
 class RemotePath(Path):
     """The class implementing remote-machine paths"""
 
-    __slots__ = ["_path", "remote"]
-    def __init__(self, remote, *parts):
+
+    def __new__(cls, remote, *parts):
         if not parts:
-            raise TypeError("At least one path part is require (none given)")
-        self.remote = remote
-        windows = (self.remote.uname.lower() == "windows")
+            raise TypeError("At least one path part is required (none given)")
+        windows = (remote.uname.lower() == "windows")
         normed = []
-        parts = (self.remote.cwd,) + parts
+        parts = (remote._session.run("pwd")[1].strip(),) + parts
         for p in parts:
             if windows:
                 plist = str(p).replace("\\", "/").split("/")
@@ -52,20 +54,25 @@ class RemotePath(Path):
                 else:
                     normed.append(item)
         if windows:
-            self.CASE_SENSITIVE = False
-            self._path = "\\".join(normed)
+            self = super(RemotePath, cls).__new__(cls, "\\".join(normed))
+            self.CASE_SENSITIVE = False # On this object only
         else:
-            self._path = "/" + "/".join(normed)
+            self = super(RemotePath, cls).__new__(cls, "/" + "/".join(normed))
+            self.CASE_SENSITIVE = True
+
+        self.remote = remote
+        return self
 
     def _form(self, *parts):
         return RemotePath(self.remote, *parts)
 
-    def __str__(self):
-        return self._path
+    @property
+    def _path(self):
+        return str(self)
 
     @property
     @_setdoc(Path)
-    def basename(self):
+    def name(self):
         if not "/" in str(self):
             return str(self)
         return str(self).rsplit("/", 1)[1]
@@ -76,16 +83,16 @@ class RemotePath(Path):
         if not "/" in str(self):
             return str(self)
         return self.__class__(self.remote, str(self).rsplit("/", 1)[0])
-        
+
     @property
     @_setdoc(Path)
     def suffix(self):
-        return '.' + self.basename.rsplit('.',1)[1]
-        
+        return '.' + self.name.rsplit('.',1)[1]
+
     @property
     @_setdoc(Path)
     def suffixes(self):
-        name = self.basename
+        name = self.name
         exts = []
         while '.' in name:
             name, ext = name.rsplit('.',1)
@@ -113,26 +120,32 @@ class RemotePath(Path):
 
     @_setdoc(Path)
     def list(self):
-        if not self.isdir():
+        if not self.is_dir():
             return []
         return [self.join(fn) for fn in self.remote._path_listdir(self)]
+        
+    @_setdoc(Path)
+    def iterdir(self):
+        if not self.is_dir():
+            return ()
+        return (self.join(fn) for fn in self.remote._path_listdir(self))
 
     @_setdoc(Path)
-    def isdir(self):
+    def is_dir(self):
         res = self.remote._path_stat(self)
         if not res:
             return False
         return res.text_mode == "directory"
 
     @_setdoc(Path)
-    def isfile(self):
+    def is_file(self):
         res = self.remote._path_stat(self)
         if not res:
             return False
         return res.text_mode in ("regular file", "regular empty file")
 
     @_setdoc(Path)
-    def islink(self):
+    def is_symlink(self):
         res = self.remote._path_stat(self)
         if not res:
             return False
@@ -152,12 +165,12 @@ class RemotePath(Path):
     @_setdoc(Path)
     def with_name(self, name):
         return self.__class__(self.remote, self.dirname) / name
-        
+
     @_setdoc(Path)
     def with_suffix(self, suffix, depth=1):
         if (suffix and not suffix.startswith('.') or suffix == '.'):
             raise ValueError("Invalid suffix %r" % (suffix))
-        name = self.basename
+        name = self.name
         depth = len(self.suffixes) if depth is None else min(depth, len(self.suffixes))
         for i in range(depth):
             name, ext = name.rsplit('.',1)
@@ -165,7 +178,8 @@ class RemotePath(Path):
 
     @_setdoc(Path)
     def glob(self, pattern):
-        return [RemotePath(self.remote, m) for m in self.remote._path_glob(self, pattern)]
+        fn = lambda pat: [RemotePath(self.remote, m) for m in self.remote._path_glob(self, pat)]
+        return self._glob(pattern, fn)
 
     @_setdoc(Path)
     def delete(self):
@@ -217,7 +231,7 @@ class RemotePath(Path):
 
     @_setdoc(Path)
     def chown(self, owner = None, group = None, recursive = None):
-        self.remote._path_chown(self, owner, group, self.isdir() if recursive is None else recursive)
+        self.remote._path_chown(self, owner, group, self.is_dir() if recursive is None else recursive)
     @_setdoc(Path)
     def chmod(self, mode):
         self.remote._path_chmod(mode, self)
@@ -250,21 +264,41 @@ class RemotePath(Path):
             raise TypeError("dst must be a string or a RemotePath (to the same remote machine), "
                 "got %r" % (dst,))
         self.remote._path_link(self, dst, True)
+    def open(self):
+        pass
 
+    @_setdoc(Path)
+    def as_uri(self, scheme = 'ssh'):
+        return '{0}://{1}{2}'.format(scheme, self.remote._fqhost, urllib.pathname2url(str(self)))
+
+    @property
+    @_setdoc(Path)
+    def stem(self):
+        return self.name.rsplit('.')[0]
+        
+    @property
+    @_setdoc(Path)
+    def root(self):
+        return '/'
+        
+    @property
+    @_setdoc(Path)
+    def drive(self):
+        return ''
 
 class RemoteWorkdir(RemotePath):
     """Remote working directory manipulator"""
 
-    def __init__(self, remote):
-        self.remote = remote
-        self._path = self.remote._session.run("pwd")[1].strip()
+    def __new__(cls, remote):
+        self = super(RemoteWorkdir, cls).__new__(cls, remote, remote._session.run("pwd")[1].strip())
+        return self
     def __hash__(self):
         raise TypeError("unhashable type")
 
     def chdir(self, newdir):
         """Changes the current working directory to the given one"""
         self.remote._session.run("cd %s" % (shquote(newdir),))
-        self._path = self.remote._session.run("pwd")[1].strip()
+        return self.__class__(self.remote)
 
     def getpath(self):
         """Returns the current working directory as a
@@ -280,9 +314,9 @@ class RemoteWorkdir(RemotePath):
                        :class:`RemotePath <plumbum.path.remote.RemotePath>`)
         """
         prev = self._path
-        self.chdir(newdir)
+        changed_dir = self.chdir(newdir)
         try:
-            yield
+            yield changed_dir
         finally:
             self.chdir(prev)
 
