@@ -1,4 +1,3 @@
-from __future__ import with_statement
 from plumbum.path.base import Path
 from plumbum.lib import six
 from plumbum.machines.local import local, LocalPath
@@ -29,8 +28,8 @@ def move(src, dst):
     """Moves the source path onto the destination path; ``src`` and ``dst`` can be either
     strings, :class:`LocalPaths <plumbum.path.local.LocalPath>` or
     :class:`RemotePath <plumbum.path.remote.RemotePath>`; any combination of the three will
-    work. 
-    
+    work.
+
     .. versionadded:: 1.3
         ``src`` can also be a list of strings/paths, in which case ``dst`` must not exist or be a directory.
     """
@@ -39,7 +38,7 @@ def move(src, dst):
     if isinstance(src, (tuple, list)):
         if not dst.exists():
             dst.mkdir()
-        elif not dst.isdir():
+        elif not dst.is_dir():
             raise ValueError("When using multiple sources, dst %r must be a directory" % (dst,))
         for src2 in src:
             move(src2, dst)
@@ -74,7 +73,7 @@ def copy(src, dst):
     if isinstance(src, (tuple, list)):
         if not dst.exists():
             dst.mkdir()
-        elif not dst.isdir():
+        elif not dst.is_dir():
             raise ValueError("When using multiple sources, dst %r must be a directory" % (dst,))
         for src2 in src:
             copy(src2, dst)
@@ -96,7 +95,7 @@ def copy(src, dst):
     else:
         with local.tempdir() as tmp:
             copy(src, tmp)
-            copy(tmp / src.basename, dst)
+            copy(tmp / src.name, dst)
         return dst
 
 
